<?php

if (!defined('ABSPATH')) {
	exit;
}

if (!class_exists('WC_Integration_Sendpulse')) :
    class WC_Integration_Sendpulse extends WC_Integration {
        private $s_api = NULL;
        private $s_api_error = '';

        /**
         * Init and hook in the integration.
         */
        public function __construct() {
            global $woocommerce;

            if (!class_exists('SendpulseClient')) {
                include_once('api/sendpulseClientInterface.php');
                include_once('api/sendpulseClient.php');
            }

            $this->id = 'sendpulse';
            $this->method_title = __('Integration WooCommerce SendPulse', 'sendpulse-for-woocommerce');
            $this->method_description = __('Sendpulse is a popular email and sms marketing service.', 'sendpulse-for-woocommerce');

            // Load the settings.
            $this->init_settings();

            // Define user set variables.
            $this->api_id = strip_tags(!empty($_POST['woocommerce_sendpulse_api_id']) ? trim($_POST['woocommerce_sendpulse_api_id']) : $this->get_option('api_id'));
            $this->api_secret = strip_tags(!empty($_POST['woocommerce_sendpulse_api_secret']) ? trim($_POST['woocommerce_sendpulse_api_secret']) : $this->get_option('api_secret'));

            // Set storage type based on admin setting
	        $this->storageType = strip_tags(!empty($_POST['woocommerce_sendpulse_keep_token']) ? trim($_POST['woocommerce_sendpulse_keep_token']) : $this->get_option('keep_token', 'wp_options'));

            // Set path to plugin's root directory
            $this->apiFilesPath = plugin_dir_path( __DIR__ );

            // Set debug mode
	        $this->debug_mode = $this->get_option('debug_mode') === 'yes';
        
            if (!empty($this->api_id) && !empty($this->api_secret)) {
                try {
                    $this->s_api = new SendpulseClient($this->api_id, $this->api_secret, $this->storageType, $this->debug_mode);
                } catch (Exception $e) {
                    $this->s_api_error = $e->getMessage();
                    add_action('admin_notices', array($this, 'get_message_api_error'));
                    add_action('network_admin_notices', array($this, 'get_message_api_error'));
                }
            }

            $this->init_form_fields();

            $this->module_status = $this->get_option('module_status');
            $this->module_status = 'yes' == $this->module_status ? 1 : 0;
            $this->mlist = $this->get_option('mlist');
            $this->subscr_time = $this->get_option('subscr_time');
            $this->show_confirm_opt = $this->get_option('show_confirm_opt');
            $this->confirm_opt_label = $this->get_option('confirm_opt_label');
            $this->confirm_opt_checkbox_default_status = $this->get_option('confirm_opt_checkbox_default_status');
            $this->confirm_opt_checkbox_display_location = $this->get_option('confirm_opt_checkbox_display_location');

            // Actions.
            add_action('woocommerce_update_options_integration_' . $this->id, array($this, 'process_admin_options'));
            add_action('woocommerce_checkout_update_order_meta', array($this, 'order_status_changed'), 1000, 1);
            add_action('woocommerce_order_status_changed', array($this, 'order_status_changed'), 10, 3);
            add_filter('woocommerce_checkout_fields', array($this, 'add_checkout_fields'));
            add_filter('default_checkout_sendpulse-for-woocommerce_confirm_opt', array($this, 'checkbox_default_status'));

            // Maybe save the "opt-in" field on the checkout
            add_action('woocommerce_checkout_update_order_meta', array($this, 'save_checkout_fields'));
        }

        public function initApiSendpulse() {
            if (!empty($this->api_id) && !empty($this->api_secret)) {
	            
                try {
                    $this->s_api = new SendpulseClient($this->api_id, $this->api_secret, $this->storageType, $this->debug_mode);
                    return true;
                } catch (Exception $e) {
                    $this->s_api_error = $e->getMessage();
                    add_action('admin_notices', array($this, 'get_message_api_error'));
                    add_action('network_admin_notices', array($this, 'get_message_api_error'));
                }
            }

            return false;
        }

        /**
         * Initialize integration settings form fields.
         */
        public function init_form_fields()  {

            if (is_admin() && !is_ajax()) {

                if ($this->s_api) {
                    $lists_opt = $this->getAddresBooksSelect();
                } else {
                    $lists_opt = array('' => __('Enter your your API data and save to see your lists'));
                }
	            //wp_nonce_field( 'clear_sendpulse_cache_action', 'clear_sendpulse_cache_nonce' );
	            $this->form_fields = array(
                    'module_status' => array(
                        'title' => __('Enable/Disable', 'sendpulse-for-woocommerce'),
                        'label' => __('Enable WooCommerce SendPulse', 'sendpulse-for-woocommerce'),
                        'type' => 'checkbox',
                        'description' => '',
                        'default' => 'no'
                    ),
                    'keep_token' => array(
	                    'title' => __('Keep token at:', 'sendpulse-for-woocommerce'),
	                    'type' => 'select',
	                    'description' => __('In case you get error message "Couldn\'t connect to API, check your ID and Secret" change your storage type to Database', 'sendpulse-for-woocommerce'),
	                    'default' => 'file',
	                    'options' => array(
		                    'file' => __('File', 'sendpulse-for-woocommerce'),
		                    'wp_options' => __('Database', 'sendpulse-for-woocommerce'),
	                    ),
                    ),
                    'api_id' => array(
                        'title' => __('API ID', 'sendpulse-for-woocommerce'),
                        'type' => 'text',
                        'description' => __("Enter with your API ID. You can find this in <a href='https://login.sendpulse.com/settings/#api'>Account settings</a> > API.", 'sendpulse-for-woocommerce'),
                        'default' => ''
                    ),
                    'api_secret' => array(
                        'title' => __('API Secret', 'sendpulse-for-woocommerce'),
                        'type' => 'text',
                        'description' => __("Enter with your API Secret. You can find this in <a href='https://login.sendpulse.com/settings/#api'>Account settings</a> > API.", 'sendpulse-for-woocommerce'),
                        'default' => ''
                    ),
                    'mlist' => array(
                        'title' => __('Mailing list', 'sendpulse-for-woocommerce'),
                        'type' => 'select',
                        'description' => __('All customers will be added to this list.', 'sendpulse-for-woocommerce'),
                        'default' => '',
                        'options' => $lists_opt,
                    ),
                    'subscr_time' => array(
                        'title' => __('Subscribe Event', 'sendpulse-for-woocommerce'),
                        'type' => 'select',
                        'description' => __('When should customers be subscribed to Mailing lists?', 'sendpulse-for-woocommerce'),
                        'default' => 'pending',
                        'options' => array(
                            'pending' => __('Order Created', 'sendpulse-for-woocommerce'),
                            'processing' => __('Order Processing', 'sendpulse-for-woocommerce'),
                            'completed' => __('Order Completed', 'sendpulse-for-woocommerce'),
                        ),
                    ),
                    'show_confirm_opt' => array(
                        'title' => __('Display Opt-In Field', 'sendpulse-for-woocommerce'),
                        'label' => __('Display an Opt-In Field on Checkout', 'sendpulse-for-woocommerce'),
                        'type' => 'checkbox',
                        'description' => __('If enabled, customers will be presented with a "Opt-in" checkbox during checkout and will only be added to the list above if they opt-in.', 'sendpulse-for-woocommerce'),
                        'default' => 'no',
                    ),
                    'confirm_opt_label' => array(
                        'title' => __('Opt-In Field Label', 'sendpulse-for-woocommerce'),
                        'type' => 'text',
                        'description' => __('Optional: customize the label displayed next to the opt-in checkbox.', 'sendpulse-for-woocommerce'),
                        'default' => __('Add me to the newsletter (we will never share your email).', 'sendpulse-for-woocommerce'),
                    ),
                    'confirm_opt_checkbox_default_status' => array(
                        'title' => __('Opt-In Checkbox Default Status', 'sendpulse-for-woocommerce'),
                        'type' => 'select',
                        'description' => __('The default state of the opt-in checkbox.', 'sendpulse-for-woocommerce'),
                        'default' => 'checked',
                        'options' => array(
                            'checked' => __('Checked', 'sendpulse-for-woocommerce'),
                            'unchecked' => __('Unchecked', 'sendpulse-for-woocommerce')
                        )
                    ),
                    'confirm_opt_checkbox_display_location' => array(
                        'title' => __('Opt-In Checkbox Display Location', 'sendpulse-for-woocommerce'),
                        'type' => 'select',
                        'description' => __('Where to display the opt-in checkbox on the checkout page (under Billing info or Order info).', 'sendpulse-for-woocommerce'),
                        'default' => 'billing',
                        'options' => array(
                            'billing' => __('Billing', 'sendpulse-for-woocommerce'),
                            'order' => __('Order', 'sendpulse-for-woocommerce')
                        )
                    ),

                );

	            $this->form_fields['debug_mode'] = array(
		            'title' => __('Debug Mode', 'sendpulse-for-woocommerce'),
		            'label' => __('Check to enable Debug Mode', 'sendpulse-for-woocommerce'),
		            'type' => 'checkbox',
		            'description' => __('Enable debug logging for troubleshooting. Make sure you set debug variables <b>WP_DEBUG</b> and <b>WP_DEBUG_LOG</b> at <i>wp-config.php</i> file to true. 
                                        <br>After you disable debug option change values you edit to false.', 'sendpulse-for-woocommerce'),
		            'default' => 'no'
	            );

                $this->wc_enqueue_js("
				jQuery('#woocommerce_sendpulse_show_confirm_opt').change(function(){
					jQuery('#mainform [id^=woocommerce_sendpulse_confirm_opt]').closest('tr').hide('fast');

					if ( jQuery(this).prop('checked') == true ) {
						jQuery('#mainform [id^=woocommerce_sendpulse_confirm_opt]').closest('tr').show('fast');
					}
					else {
						jQuery('#mainform [id^=woocommerce_sendpulse_confirm_opt]').closest('tr').hide('fast');
					}

				}).change();
			");
            }
        }

	    public function process_admin_options() {
		    $previous_keep_token = $this->get_option('keep_token', 'wp_options');
		    parent::process_admin_options();

		    // Save the keep_token option to the database
		    $new_keep_token = $_POST['woocommerce_sendpulse_keep_token'];
		    update_option('woocommerce_sendpulse_keep_token', $new_keep_token);

		    // If the storage type has changed, delete the token from the previous storage
		    if ($previous_keep_token !== $new_keep_token) {
			    $this->deleteTokenFromStorage($previous_keep_token);
		    }

		    $this->storageType = $new_keep_token;
	    }

	    private function deleteTokenFromStorage($storageType) {
		    $hashName = md5($this->api_id . '::' . $this->api_secret);
		    switch ($storageType) {
			    case 'file':
				    $this->deleteFileToken($hashName);
				    break;
			    case 'wp_options':
				    $this->deleteDbToken();
				    break;
		    }
	    }

	    private function deleteFileToken($hashName) {
		    $tokenFilePath = $this->apiFilesPath . $hashName;
		    if (file_exists($tokenFilePath)) {
			    unlink($tokenFilePath);

		    }
	    }

	    private function deleteDbToken() {
		    update_option('woocommerce_sendpulse_token', '');
	    }

	    public function getAddresBooksSelect() {
		    //$cache_key = 'sendpulse_address_books';
		    //$cache_expiration = 3600; // Cache for 1 hour

		    // Try to get the cached address books
		    //$cached_abooks = get_transient($cache_key);
		    //if ($cached_abooks !== false) {
			//    return $cached_abooks;
		    //}

            $abooks = array('' => __('Select a list...', 'sendpulse-for-woocommerce'));
            try {
                $abooks_ = $this->s_api->listAddressBooks();

	            if (isset($abooks_->http_code) && $abooks_->http_code == 429) {
		            // Handle rate limiting error
		            $this->s_api_error = __('Too many requests. Error code: 429. See more: https://sendpulse.com/integrations/api#limits', 'sendpulse-for-woocommerce');
		            add_action('admin_notices', array($this, 'get_message_api_error'));
		            add_action('network_admin_notices', array($this, 'get_message_api_error'));
		            return $abooks;
	            }

                if ($this->debug_mode) {
		            error_log( 'getAddresBooksSelect response: ' . print_r( $abooks_, true ) );
	            }

                if (!empty($abooks_)) {
                    foreach ($abooks_ as $abook) {
                        $abooks[$abook->id . ""] = mb_strimwidth($abook->name, 0, 100, "...") . ' (' . $abook->all_email_qty . ')';
                    }

	                // Cache the response
	                //set_transient($cache_key, $abooks, $cache_expiration);
                }
            } catch (Exception $e) {
                $this->s_api_error = $e->getMessage();
                add_action('admin_notices', array($this, 'get_message_api_error'));
                add_action('network_admin_notices', array($this, 'get_message_api_error'));
                return false;
            }

            return $abooks;
        }

	    private function get_message($message, $type = 'error') {
            ob_start();
            ?>
            <div class="<?php echo $type ?>">
                <p><?php echo $message ?></p>
            </div>
            <?php
            return ob_get_clean();
        }

        public function get_message_api_error() {
            echo $this->get_message($this->s_api_error);
        }

        public function syncUserAbook($email, $first_name, $last_name, $phone, $company, $country, $state, $city) {

            if (!$this->s_api || empty($email) || !$this->module_status || empty($this->mlist)) {
                return false;
            }
                
            $ab_id = $this->mlist;
            $countries_obj = new WC_Countries();
            $countries_array = $countries_obj->get_countries();
            $country_states_array = $countries_obj->get_states();

            try {
                $user_data = array('email' => $email);
                $variables = array();
                if (!empty($phone)) {
                    $variables['phone'] = trim($phone);
                }
                if (!empty($first_name)) {
                    $variables['name'] = trim($first_name);
                }
                if (!empty($last_name)) {
                    $variables['name'] .= ' ' . trim($last_name);
                }
                if (!empty($company)) {
                    $variables['company'] = trim($company);
                }
                if (!empty($country)) {
                    $country_name = $countries_array[trim($country)];
                    $variables['country'] = trim($country_name);
                }
                if (!empty($state)) {
                    $country_name = $countries_array[trim($country)];
                    $state_name = $country_states_array[trim($country)][trim($state)];
                    $variables['state'] = trim($state_name);
                }
                if (!empty($city)) {
                    $variables['city'] = trim($city);
                }
                if (!empty($variables))
                    $user_data['variables'] = $variables;

                return $this->s_api->addEmails($ab_id, array($user_data));
            } catch (Exception $e) {
                $this->s_api_error = $e->getMessage();
                add_action('network_admin_notices', array($this, 'get_message_api_error'));
                return false;
            }
        }

        function add_checkout_fields($checkout_fields) {
            if (!$this->module_status)
                return $checkout_fields;

            $display_location = $this->confirm_opt_checkbox_display_location;

            if (empty($display_location)) {
                $display_location = 'billing';
            }

            if ($this->show_confirm_opt == 'yes') {
                $checkout_fields[$display_location]['sendpulse-for-woocommerce_confirm_opt'] = array(
                    'type' => 'checkbox',
                    'label' => esc_attr($this->confirm_opt_label),
                    'default' => $this->confirm_opt_checkbox_default_status == 'checked' ? 1 : 0,
                );
            }

            return $checkout_fields;
        }

        public function order_status_changed($id, $old_status = 'new', $new_status = 'pending') {

            if ($this->s_api && $this->module_status && $new_status == $this->subscr_time) {
                // Get WC order
                $order = $this->wc_get_order($id);

	            // Log the order status change
	            if ($this->debug_mode) {
		            error_log("Order status changed for order $id: $old_status -> $new_status");
	            }

                $subscribe_customer = get_post_meta($id, 'sendpulse-for-woocommerce_confirm_opt', true);
                if (!$subscribe_customer || empty($subscribe_customer) || $subscribe_customer == 'yes') {

                    $this->syncUserAbook(
                        $order->billing_email, 
                        $order->billing_first_name, 
                        $order->billing_last_name, 
                        $order->billing_phone,
                        $order->billing_company,
                        $order->billing_country,
                        $order->billing_state,
                        $order->billing_city
                    );
                }
            }

        }

        private function wc_get_order($order_id) {
            if (function_exists('wc_get_order')) {
                return wc_get_order($order_id);
            } else {
                return new WC_Order($order_id);
            }
        }

        function save_checkout_fields($order_id) {
            if ('yes' == $this->show_confirm_opt) {
                $opt_in = isset($_POST['sendpulse-for-woocommerce_confirm_opt']) ? 'yes' : 'no';
                update_post_meta($order_id, 'sendpulse-for-woocommerce_confirm_opt', $opt_in);
            }
        }

        function checkbox_default_status($input) {
            return $this->confirm_opt_checkbox_default_status == 'checked' ? 1 : 0;
        }

        private function wc_enqueue_js($code) {
            if (function_exists('wc_enqueue_js')) {
                wc_enqueue_js($code);
            } else {
                global $woocommerce;
                $woocommerce->add_inline_js($code);
            }
        }

        private function _debug($array, $exit = 0) {
            echo "<pre>";
            var_dump($array);
            echo "</pre>";
            if ($exit) {
                exit();
            }
        }

    }

endif;